"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var XhrInterceptor_1 = __importDefault(require("./XhrInterceptor"));
var container = { instance: null };
var XhrResponseMock = /** @class */ (function () {
    function XhrResponseMock() {
        if (container.instance)
            return container.instance;
        container.instance = this;
        this.interceptor = new XhrInterceptor_1.default();
        return this;
    }
    XhrResponseMock.init = function () {
        return new XhrResponseMock();
    };
    XhrResponseMock.mockXMLHttpRequest = function () {
        window.XMLHttpRequest = function () { };
        window.XMLHttpRequest.prototype = {
            open: function () { },
            send: function () { },
            setRequestHeader: function () { },
            onreadystatechange: function () { },
            load: function () { },
            loadend: function () { },
            get readyState() { return 4; },
            get status() { return 200; },
            get statusText() { return ''; },
            get response() { return ''; },
            get responseText() { return ''; },
        };
    };
    XhrResponseMock.prototype.doMock = function (mockItem) {
        if (!mockItem.url || typeof mockItem.url !== 'string' && !(mockItem.url instanceof RegExp)) {
            return this;
        }
        if (mockItem.data === undefined) {
            return this;
        }
        mockItem.method = /^(get|post|put|patch|delete|any)$/i.test(mockItem.method)
            ? mockItem.method.toLowerCase()
            : 'any';
        var key = mockItem.url + "-" + mockItem.method;
        this.interceptor.addMockData(key, mockItem);
        return this;
    };
    XhrResponseMock.prototype.setMockData = function (mockData) {
        this.interceptor.setMockData(mockData);
        return this;
    };
    XhrResponseMock.prototype.reset = function () {
        this.setMockData({});
        return this;
    };
    XhrResponseMock.prototype.get = function (url, mockData, delay) {
        if (delay === void 0) { delay = 0; }
        this.doMock({ url: url, method: 'get', data: mockData, delay: delay });
        return this;
    };
    XhrResponseMock.prototype.post = function (url, mockData, delay) {
        if (delay === void 0) { delay = 0; }
        this.doMock({ url: url, method: 'post', data: mockData, delay: delay });
        return this;
    };
    XhrResponseMock.prototype.put = function (url, mockData, delay) {
        if (delay === void 0) { delay = 0; }
        this.doMock({ url: url, method: 'put', data: mockData, delay: delay });
        return this;
    };
    XhrResponseMock.prototype.patch = function (url, mockData, delay) {
        if (delay === void 0) { delay = 0; }
        this.doMock({ url: url, method: 'patch', data: mockData, delay: delay });
        return this;
    };
    XhrResponseMock.prototype.delete = function (url, mockData, delay) {
        if (delay === void 0) { delay = 0; }
        this.doMock({ url: url, method: 'delete', data: mockData, delay: delay });
        return this;
    };
    XhrResponseMock.prototype.any = function (url, mockData, delay) {
        if (delay === void 0) { delay = 0; }
        this.doMock({ url: url, method: 'any', data: mockData, delay: delay });
        return this;
    };
    return XhrResponseMock;
}());
exports.default = XhrResponseMock;
//# sourceMappingURL=XhrResponseMock.js.map